﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using StriveEngine.Tcp.Server;
using StriveEngine;
using Core;
using System.Net;
using ESBasic;
using System.Threading;
using ESBasic.Helpers;
using ESBasic.ObjectManagement.Managers;
using System.Configuration;

namespace Server
{
    public partial class FormServer : Form
    {
        private ITcpServerEngine tcpServerEngine;
        private ObjectManager<string, UserInfo> onlineManager = new ObjectManager<string,UserInfo>(); //在线用户管理器    
        /// <summary>
        /// 构造方法
        /// </summary>
        public FormServer()
        {
            InitializeComponent();
            this.tcpServerEngine = NetworkEngineFactory.CreateStreamTcpServerEngine(int.Parse(ConfigurationManager.AppSettings["Port"]), new StreamContractHelper());
            this.tcpServerEngine.MessageReceived += new CbDelegate<IPEndPoint, byte[]>(tcpServerEngine_MessageReceived);
            this.tcpServerEngine.ClientDisconnected += new CbDelegate<IPEndPoint>(tcpServerEngine_ClientDisconnected);
            this.tcpServerEngine.Initialize();          
        }
        /// <summary>
       /// 客户端掉线事件处理
       /// </summary>
       /// <param name="userAddress">掉线客户端IP端口号</param>
        void tcpServerEngine_ClientDisconnected(IPEndPoint userAddress)
        {           
            foreach (string  currentUserID in this.onlineManager.GetKeyList())
            {
                if (this.onlineManager.Get(currentUserID).Address == userAddress)
                {
                    this.RemoveUser(currentUserID);
                    this.ShowOnlineUserCount();
                    this.TellOtherUser(MsgType.FriendOffline,currentUserID);
                }
            }            
        }

        #region 接收消息处理
        /// <summary>
        /// 消息接收事件处理
        /// </summary>
        /// <param name="userAddress">消息源用户IP端口号</param>
        /// <param name="msg">消息</param>
        void tcpServerEngine_MessageReceived(IPEndPoint userAddress, byte[] msg)
        {
            //取出消息类型
            int msgType = BitConverter.ToInt32(msg, 0);
            //验证消息类型
            if (msgType == MsgType.Logining)
            {
                LoginMsg loginMsg = MsgHelper.DeserializeMsg<LoginMsg>(msg);
                this.ReplyLogining(loginMsg, userAddress);
                //将在线用户告知其他客户端                
                this.TellOtherUser(MsgType.NewOnlineFriend, loginMsg.SrcUserID);
            }
            if (msgType == MsgType.InitializeFriendList)
            {
                FriendListMsg friendListMsg = MsgHelper.DeserializeMsg<FriendListMsg>(msg);
                List<string> onlineList = this.onlineManager.GetKeyList();
                onlineList.Remove(friendListMsg.SelfID);
                friendListMsg.FriendList = onlineList;
                this.tcpServerEngine.SendMessageToClient(userAddress, friendListMsg.ToContractStream());
            }

            if (msgType == MsgType.Chatting)
            {
                ChatMsg chatMsg = MsgHelper.DeserializeMsg<ChatMsg>(msg);
                if (this.onlineManager.GetKeyList().Contains(chatMsg.TargetUserID))
                {
                    IPEndPoint targetUserAddress = this.onlineManager.Get(chatMsg.TargetUserID).Address;
                    this.tcpServerEngine.SendMessageToClient(targetUserAddress, msg);
                }
            }
        } 
        #endregion

        #region 回复登陆消息
        /// <summary>
        /// 回复登陆消息
        /// </summary>
        /// <param name="loginMsg"></param>
        /// <param name="userAddress"></param>
        private void ReplyLogining(LoginMsg loginMsg, IPEndPoint userAddress)
        {
            if (this.onlineManager.Contains(loginMsg.SrcUserID))//重复登录
            {
                loginMsg.LogonResult = LogonResult.Repetition;
                this.tcpServerEngine.SendMessageToClient(userAddress, loginMsg.ToContractStream());
            }
            else//此demo简化处理回复成功，其他验证未处理
            {
                this.AddUser(loginMsg.SrcUserID, userAddress);
                this.ShowOnlineUserCount();
                loginMsg.LogonResult = LogonResult.Succeed;
                this.tcpServerEngine.SendMessageToClient(userAddress, loginMsg.ToContractStream());
            }
        } 
        #endregion
   
        #region 用户添加与删除
        /// <summary>
        /// 添加新用户
        /// </summary>
        /// <param name="logonUserID">待添加用户ID</param>
        /// <param name="userAddress">待添加用户IP端口号</param>
        private void AddUser(string logonUserID, IPEndPoint userAddress)
        {
            UserInfo info = new UserInfo(logonUserID, userAddress);
            this.onlineManager.Add(logonUserID, info);
            this.AddItem(info);
        }
        /// <summary>
        /// 移除用户
        /// </summary>
        /// <param name="userID">待移除用户ID</param>
        private void RemoveUser(string userID)
        {
            this.RemoveItem(this.onlineManager.Get(userID));
            this.onlineManager.Remove(userID);
        }
        /// <summary>
        /// 删除下线用户UI显示
        /// </summary>
        /// <param name="info">用户信息</param>
        private void RemoveItem(UserInfo info)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new CbGeneric<UserInfo>(this.RemoveItem), info);
            }
            else
            {
                this.listView1.Items.Remove(info.ListViewItem);
            }
        }
        /// <summary>
        ///添加上线用户UI显示
        /// </summary>
        /// <param name="info">用户信息</param>
        private void AddItem(UserInfo info)
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new CbGeneric<UserInfo>(this.AddItem), info);
            }
            else
            {
                string[] subItems = { info.UserID, info.AddressText, info.LogonTimeText };
                ListViewItem item = new ListViewItem(subItems);
                info.ListViewItem = item;
                this.listView1.Items.Add(item);
            }
        } 
        #endregion

        #region 同步新上线用户给其他用户
        /// <summary>
        /// 告知其他在线用户
        /// </summary>
        /// <param name="msgType">消息类型</param>
        /// <param name="currentUserID">当前用户ID</param>
        private void TellOtherUser(int msgType, string currentUserID)
        {
            List<string> listOtherUser = this.GetOtherUserList(currentUserID);
            foreach (string key in listOtherUser)
            {
                this.tcpServerEngine.SendMessageToClient(this.onlineManager.Get(key).Address,
                                                  MsgHelper.BuildMsg(msgType,
                                                                           Encoding.UTF8.GetBytes(currentUserID)));

            }
        }

        private List<string> GetOtherUserList(string currentUserID)
        {
            List<string> listOtherUser = this.onlineManager.GetKeyList();          
            listOtherUser.Remove(currentUserID);           
            return listOtherUser;
        } 
        #endregion
        
        /// <summary>
        /// 显示在线用户数量
        /// </summary>     
        private void ShowOnlineUserCount()
        {
            if (this.InvokeRequired)
            {
                this.Invoke(new CbGeneric(this.ShowOnlineUserCount), null);
            }
            else
            {
                this.toolStripLabel_UserCount.Text = "在线人数：" + this.onlineManager.GetKeyList().Count.ToString();
            }
        }

        private void FormServer_Shown(object sender, EventArgs e)
        {
            this.toolStripLabel_Port.Text = "监听端口：" + ConfigurationManager.AppSettings["Port"];
        }
    }
}
